/**
 ***********************************************************************
 * $Id: Lib.cs 330 2015-01-06 08:08:07Z mkwayisi $
 * ---------------------------------------------------------------------
 * Authored by Michael Kwayisi. Copyright (c) 2014. See license below.
 * Comments are appreciated - mic at kwayisi dot org | www.kwayisi.org
 * ---------------------------------------------------------------------
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are stringently met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions, and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions, and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. The end-user documentation included with the redistribution,
 *     if any, must include the following acknowledgment:
 *       "This product includes software written by Michael Kwayisi."
 *     Alternately, this acknowledgment may appear in the software
 *     itself, if and wherever such 3rd-party acknowledgments appear.
 *  4. Neither the name of the software nor the name of its author
 *     and/or contributors may be used to endorse or promote products
 *     derived from this software without specific prior permission.
 ***********************************************************************
 */

using System;

public static class Lib
{
//======================================================================
// [DATA] Control drawing characters. S= Single; D= Double; T= Top;
// B= Bottom; L= Left; R= Right; C= Center; VT= Vertical; HZ= Horizontal;
public const char CharSLT = (char) 0x250c;
public const char CharSLC = (char) 0x251c;
public const char CharSLB = (char) 0x2514;
public const char CharSRT = (char) 0x2510;
public const char CharSRC = (char) 0x2524;
public const char CharSRB = (char) 0x2518;
public const char CharDLT = (char) 0x2554;
public const char CharDLB = (char) 0x255a;
public const char CharDLC = (char) 0x2560;
public const char CharDRT = (char) 0x2557;
public const char CharDRC = (char) 0x2563;
public const char CharDRB = (char) 0x255d;
public const char CharSHZ = (char) 0x2500;
public const char CharSVT = (char) 0x2502;
public const char CharDHZ = (char) 0x2550;
public const char CharDVT = (char) 0x2551;

//======================================================================
// [FUNC] Reads the next line of "valid" characters from the standard
// input stream. "Valid" here means that a trimmed input stream contents
// contain at least an ASCII character between the range 0x20 and 0x7f.
public static string ReadLine(int maxChars)
{
	bool isDone = false;
	string buffer = "";
	int cursorLeft = Console.CursorLeft;
	ConsoleKeyInfo keyInfo;
	
	Console.Write(CharSVT.ToString());
	do {
		keyInfo = Console.ReadKey(true);
		switch (keyInfo.Key) {
			case ConsoleKey.Escape:
				if (buffer.Length > 0) {
					Console.CursorLeft = cursorLeft;
					Console.Write("".PadRight(buffer.Length + 1, ' '));
					Console.CursorLeft = cursorLeft;
					Console.Write(CharSVT);
					buffer = "";
				}
				break;
			case ConsoleKey.Backspace:
				if (buffer.Length > 0) {
					buffer = buffer.Remove(buffer.Length - 1);
					Console.CursorLeft -= 2;
					Console.Write("{0}{1}", CharSVT, ' ');
					Console.CursorLeft--;
				}
				break;
			case ConsoleKey.Enter:
				if (buffer.Length > 0)
					isDone = true;
				break;
			default:
				if (keyInfo.KeyChar >= (char) 0x20 &&
					keyInfo.KeyChar <= (char) 0x7f &&
					buffer.Length < maxChars &&
					!(buffer.Length == 0 && keyInfo.KeyChar == ' '))
				{
					buffer += keyInfo.KeyChar.ToString();
					Console.CursorLeft--;
					Console.Write("{0}{1}", keyInfo.KeyChar, CharSVT);
				}
				break;
		}
	} while (!isDone);
	
	return buffer.Trim();
}

//======================================================================
// [FUNC] Returns a "map" for a given character. 'short' is used because
// I want to refer to the characters by their unicode hex-code which C#
// can't convert to 'char' implicitly (see why C is a champion?) and
// also it has the same storage size (sometimes) as 'char'.
public static short[][] GetCharMap(char chr)
{
	switch (Char.ToUpper(chr)) {
		case 'A': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x2560, 0x2550, 0x2563},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'B': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2560, 0x2550, 0x2563},
			new short[] {0x2569, 0x2550, 0x255d}
		};
		case 'C': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x0020},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'D': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x2551},
			new short[] {0x2569, 0x2550, 0x255d}
		};
		case 'E': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2560, 0x2550, 0x0020},
			new short[] {0x2569, 0x2550, 0x255d}
		};
		case 'F': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2560, 0x2550, 0x0020},
			new short[] {0x2569, 0x0020, 0x0020}
		};
		case 'G': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x2557},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'H': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x2560, 0x2550, 0x2563},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'I': return new short[][] {
			new short[] {0x2566},
			new short[] {0x2551},
			new short[] {0x2569}
		};
		case 'J': return new short[][] {
			new short[] {0x0020, 0x2550, 0x2566},
			new short[] {0x0020, 0x0020, 0x2551},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'K': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x2560, 0x003c, 0x0020},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'L': return new short[][] {
			new short[] {0x2566, 0x0020, 0x0020},
			new short[] {0x2551, 0x0020, 0x0020},
			new short[] {0x2569, 0x2550, 0x255d}
		};
		case 'M': return new short[][] {
			new short[] {0x2566, 0x2566, 0x2566},
			new short[] {0x2551, 0x2551, 0x2551},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'N': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x2551},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'O': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x2551},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'P': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2560, 0x2550, 0x255d},
			new short[] {0x2569, 0x0020, 0x0020}
		};
		case 'Q': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x2551},
			new short[] {0x255a, 0x2550, 0x256c}
		};
		case 'R': return new short[][] {
			new short[] {0x2566, 0x2550, 0x2557},
			new short[] {0x2551, 0x0020, 0x0020},
			new short[] {0x2569, 0x0020, 0x0020}
		};
		case 'S': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x255a, 0x2550, 0x2557},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'T': return new short[][] {
			new short[] {0x2554, 0x2566, 0x2557},
			new short[] {0x0020, 0x2551, 0x0020},
			new short[] {0x0020, 0x2569, 0x0020}
		};
		case 'U': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x2551, 0x0020, 0x2551},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case 'V': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x005c, 0x0020, 0x002f},
			new short[] {0x0020, 0x2550, 0x0020}
		};
		case 'W': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x2551, 0x2551, 0x2551},
			new short[] {0x2569, 0x2569, 0x2569}
		};
		case 'X': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x0020, 0x0058, 0x0020},
			new short[] {0x2569, 0x0020, 0x2569}
		};
		case 'Y': return new short[][] {
			new short[] {0x2566, 0x0020, 0x2566},
			new short[] {0x255a, 0x2566, 0x255d},
			new short[] {0x0020, 0x2569, 0x0020}
		};
		case 'Z': return new short[][] {
			new short[] {0x2554, 0x2550, 0x2557},
			new short[] {0x0020, 0x002f, 0x0020},
			new short[] {0x255a, 0x2550, 0x255d}
		};
		case ' ': return new short[][] {
			new short[] {0x0020},
			new short[] {0x0020},
			new short[] {0x0020},
		};
		case '!': return new short[][] {
			new short[] {0x2565},
			new short[] {0x2551},
			new short[] {0x25a0}
		};
		case '-': return new short[][] {
			new short[] {0x0020, 0x0020},
			new short[] {0x2550, 0x2550},
			new short[] {0x0020, 0x0020}
		};
		default: return new short[][] {
			new short[] {},
			new short[] {},
			new short[] {}
		};
	}
}

//======================================================================
// [FUNC] Returns a "map" for a passed string. This function does the
// explicit conversion from 'short' (returned by GetCharMa") to 'char'.
public static string[] GetStringMap(string str)
{
	short[][] cmap;
	string[] smap = new string[3];
	for (int i = 0; i < str.Length; i++) {
		cmap = GetCharMap(str[i]);
		for (int j = 0; j < cmap.Length; j++) {
			for (int k = 0; k < cmap[j].Length; k++) {
				smap[j] += ((char) cmap[j][k]).ToString();
			}
		}
	}
	return smap;
}

//======================================================================
// [FUNC] Returns a "map" for given Token's value. Not anything special
// except carefully placed characters to visually represent a number!
public static string[] GetTokenMap(int val)
{
	switch (val) {
		case 0: case 9: return new string[] {"011110", "110011", "110011", "110011", "011110"};
		case 1: return new string[] {"111100", "001100", "001100", "001100", "111111"};
		case 2: return new string[] {"111110", "000011", "011110", "110000", "111111"};
		case 3: return new string[] {"111110", "000011", "011110", "000011", "111110"};
		case 4: return new string[] {"001110", "011110", "110110", "111111", "000110"};
		case 5: return new string[] {"111110", "110000", "111110", "000011", "111110"};
		case 6: return new string[] {"011110", "110000", "111110", "110011", "011110"};
		case 7: return new string[] {"111111", "000011", "000110", "001100", "001100"};
		case 8: return new string[] {"011110", "110011", "011110", "110011", "011110"};
		default: return new string[] {};
	}
}

//======================================================================
// [FUNC] Appends an affix to a number. Graham Barr calls that an
// "ornate" but I don't think it's official. The 'st', 'nd', 'rd', etc.
public static string Affixate(int num)
{
	if (num >= 11 && num <= 13)
		return num + "TH";
	else switch (num % 10) {
		case 1:  return num + "ST";
		case 2:  return num + "ND";
		case 3:  return num + "RD";
		default: return num + "TH";
	}
}
}
